@@ -97,10 +97,10 @@ void PackageDetails::init(PackageKit::Transaction::Roles roles)
         ui->descriptionW->setWidgetResizable(true);
     }
 
-    if (roles & PackageKit::Transaction::RoleGetDepends) {
+    if (roles & PackageKit::Transaction::RoleDependsOn) {
         action = menu->addAction(i18n("Depends On"));
         action->setCheckable(true);
-        action->setData(PackageKit::Transaction::RoleGetDepends);
+        action->setData(PackageKit::Transaction::RoleDependsOn);
         m_actionGroup->addAction(action);
         // Sets a transparent background
         QWidget *actionsViewport = ui->dependsOnLV->viewport();
@@ -126,10 +126,10 @@ void PackageDetails::init(PackageKit::Transaction::Roles roles)
         ui->dependsOnLV->header()->hideSection(PackageModel::SizeCol);
     }
 
-    if (roles & PackageKit::Transaction::RoleGetRequires) {
+    if (roles & PackageKit::Transaction::RoleRequiredBy) {
         action = menu->addAction(i18n("Required By"));
         action->setCheckable(true);
-        action->setData(PackageKit::Transaction::RoleGetRequires);
+        action->setData(PackageKit::Transaction::RoleRequiredBy);
         m_actionGroup->addAction(action);
         // Sets a transparent background
         QWidget *actionsViewport = ui->requiredByLV->viewport();
@@ -324,8 +324,8 @@ void PackageDetails::actionActivated(QAction *action)
     // disconnect the transaction
     // so that we don't get old data
     if (m_transaction) {
-        disconnect(m_transaction, SIGNAL(details(QString,QString,PackageKit::Transaction::Group,QString,QString,qulonglong)),
-                   this, SLOT(description(QString,QString,PackageKit::Transaction::Group,QString,QString,qulonglong)));
+        disconnect(m_transaction, SIGNAL(details(PackageKit::Details)),
+                   this, SLOT(description(PackageKit::Details)));
         disconnect(m_transaction, SIGNAL(package(PackageKit::Transaction::Info,QString,QString)),
                    m_dependsModel, SLOT(addPackage(PackageKit::Transaction::Info,QString,QString)));
         disconnect(m_transaction, SIGNAL(package(PackageKit::Transaction::Info,QString,QString)),
@@ -342,23 +342,18 @@ void PackageDetails::actionActivated(QAction *action)
     switch (role) {
     case PackageKit::Transaction::RoleGetDetails:
         if (m_hasDetails) {
-            description(m_detailsPackageID,
-                        m_detailsLicense,
-                        m_detailsGroup,
-                        m_detailsDetail,
-                        m_detailsUrl,
-                        m_detailsSize);
+            description(m_details);
             display();
             return;
         }
         break;
-    case PackageKit::Transaction::RoleGetDepends:
+    case PackageKit::Transaction::RoleDependsOn:
         if (m_hasDepends) {
             display();
             return;
         }
         break;
-    case PackageKit::Transaction::RoleGetRequires:
+    case PackageKit::Transaction::RoleRequiredBy:
         if (m_hasRequires) {
             display();
             return;
@@ -373,51 +368,44 @@ void PackageDetails::actionActivated(QAction *action)
     }
 
     // we don't have the data
-    m_transaction = new PackageKit::Transaction(this);
     kDebug() << "New transaction";
-    connect(m_transaction, SIGNAL(finished(PackageKit::Transaction::Exit,uint)),
-            this, SLOT(finished()));
     switch (role) {
     case PackageKit::Transaction::RoleGetDetails:
-        connect(m_transaction, SIGNAL(details(QString,QString,PackageKit::Transaction::Group,QString,QString,qulonglong)),
-                this, SLOT(description(QString,QString,PackageKit::Transaction::Group,QString,QString,qulonglong)));
-        m_transaction->getDetails(m_packageID);
+        m_transaction = Daemon::getDetails(m_packageID);
+        connect(m_transaction, SIGNAL(details(PackageKit::Details)),
+                SLOT(description(PackageKit::Details)));
         break;
-    case PackageKit::Transaction::RoleGetDepends:
+    case PackageKit::Transaction::RoleDependsOn:
         m_dependsModel->clear();
+        m_transaction = Daemon::dependsOn(m_packageID, PackageKit::Transaction::FilterNone, false);
         connect(m_transaction, SIGNAL(package(PackageKit::Transaction::Info,QString,QString)),
                 m_dependsModel, SLOT(addPackage(PackageKit::Transaction::Info,QString,QString)));
         connect(m_transaction, SIGNAL(finished(PackageKit::Transaction::Exit,uint)),
                 m_dependsModel, SLOT(finished()));
-        m_transaction->getDepends(m_packageID, PackageKit::Transaction::FilterNone, false);
         break;
-    case PackageKit::Transaction::RoleGetRequires:
+    case PackageKit::Transaction::RoleRequiredBy:
         m_requiresModel->clear();
+        m_transaction = Daemon::requiredBy(m_packageID, PackageKit::Transaction::FilterNone, false);
         connect(m_transaction, SIGNAL(package(PackageKit::Transaction::Info,QString,QString)),
                 m_requiresModel, SLOT(addPackage(PackageKit::Transaction::Info,QString,QString)));
         connect(m_transaction, SIGNAL(finished(PackageKit::Transaction::Exit,uint)),
                 m_requiresModel, SLOT(finished()));
-        m_transaction->getRequires(m_packageID, PackageKit::Transaction::FilterNone, false);
         break;
     case PackageKit::Transaction::RoleGetFiles:
         m_currentFileList.clear();
+        m_transaction = Daemon::getFiles(m_packageID);
         connect(m_transaction, SIGNAL(files(QString,QStringList)),
                 this, SLOT(files(QString,QStringList)));
-        m_transaction->getFiles(m_packageID);
         break;
+    default:
+        qWarning() << Q_FUNC_INFO << "Oops, unhandled role, please report" << role;
+        return;
     }
+    connect(m_transaction, SIGNAL(finished(PackageKit::Transaction::Exit,uint)),
+            this, SLOT(finished()));
     kDebug() <<"transaction running";
 
-    PackageKit::Transaction::InternalError error = m_transaction->internalError();
-    if (error) {
-        disconnect(m_transaction, SIGNAL(finished(PackageKit::Transaction::Exit,uint)),
-                   this, SLOT(finished()));
-        m_transaction = 0;
-        kDebug() << "transaction running" << error << PkStrings::daemonError(error);
-        KMessageBox::sorry(this, PkStrings::daemonError(error));
-    } else {
-        m_busySeq->start();
-    }
+    m_busySeq->start();
 }
 
 void PackageDetails::resultJob(KJob *job)
@@ -486,7 +474,7 @@ void PackageDetails::display()
                     fadeIn = true;
                 }
                 break;
-            case PackageKit::Transaction::RoleGetDepends:
+            case PackageKit::Transaction::RoleDependsOn:
                 if (m_hasDepends) {
                     if (ui->stackedWidget->currentWidget() != ui->pageDepends) {
                         ui->stackedWidget->setCurrentWidget(ui->pageDepends);
@@ -494,7 +482,7 @@ void PackageDetails::display()
                     fadeIn = true;
                 }
                 break;
-            case PackageKit::Transaction::RoleGetRequires:
+            case PackageKit::Transaction::RoleRequiredBy:
                 if (m_hasRequires) {
                     if (ui->stackedWidget->currentWidget() != ui->pageRequired) {
                         ui->stackedWidget->setCurrentWidget(ui->pageRequired);
@@ -565,16 +553,16 @@ void PackageDetails::setupDescription()
         ui->iconL->clear();
     }
 
-    if (!m_detailsDetail.isEmpty()) {
-        ui->descriptionL->setText(m_detailsDetail.replace('\n', "<br>"));
+    if (!m_detailsDescription.isEmpty()) {
+        ui->descriptionL->setText(m_detailsDescription.replace('\n', "<br>"));
         ui->descriptionL->show();
     } else {
         ui->descriptionL->clear();
     }
 
-    if (!m_detailsUrl.isEmpty()) {
-        ui->homepageL->setText("<a href=\"" + m_detailsUrl + "\">" +
-                               m_detailsUrl + "</a>");
+    if (!m_details.url().isEmpty()) {
+        ui->homepageL->setText("<a href=\"" + m_details.url() + "\">" +
+                               m_details.url() + "</a>");
         ui->homepageL->show();
     } else {
         ui->homepageL->hide();
@@ -613,31 +601,31 @@ void PackageDetails::setupDescription()
 // //                     + "</td></tr>";
 //     }
 
-    if (!m_detailsLicense.isEmpty() && m_detailsLicense != "unknown") {
+    if (!m_details.license().isEmpty() && m_details.license() != "unknown") {
         // We have a license, check if we have and should show show package version
-        if (!m_hideVersion && !Transaction::packageVersion(m_detailsPackageID).isEmpty()) {
-            ui->licenseL->setText(Transaction::packageVersion(m_detailsPackageID) + " - " + m_detailsLicense);
+        if (!m_hideVersion && !Transaction::packageVersion(m_details.packageId()).isEmpty()) {
+            ui->licenseL->setText(Transaction::packageVersion(m_details.packageId()) + " - " + m_details.license());
         } else {
-            ui->licenseL->setText(m_detailsLicense);
+            ui->licenseL->setText(m_details.license());
         }
         ui->licenseL->show();
     } else if (!m_hideVersion) {
-        ui->licenseL->setText(Transaction::packageVersion(m_detailsPackageID));
+        ui->licenseL->setText(Transaction::packageVersion(m_details.packageId()));
         ui->licenseL->show();
     } else {
         ui->licenseL->hide();
     }
 
-    if (m_detailsSize > 0) {
-        QString size = KGlobal::locale()->formatByteSize(m_detailsSize);
-        if (!m_hideArch && !Transaction::packageArch(m_detailsPackageID).isEmpty()) {
-            ui->sizeL->setText(size % QLatin1String(" (") % Transaction::packageArch(m_detailsPackageID) % QLatin1Char(')'));
+    if (m_details.size() > 0) {
+        QString size = KGlobal::locale()->formatByteSize(m_details.size());
+        if (!m_hideArch && !Transaction::packageArch(m_details.packageId()).isEmpty()) {
+            ui->sizeL->setText(size % QLatin1String(" (") % Transaction::packageArch(m_details.packageId()) % QLatin1Char(')'));
         } else {
             ui->sizeL->setText(size);
         }
         ui->sizeL->show();
-    } else if (!m_hideArch && !Transaction::packageArch(m_detailsPackageID).isEmpty()) {
-        ui->sizeL->setText(Transaction::packageArch(m_detailsPackageID));
+    } else if (!m_hideArch && !Transaction::packageArch(m_details.packageId()).isEmpty()) {
+        ui->sizeL->setText(Transaction::packageArch(m_details.packageId()));
     } else {
         ui->sizeL->hide();
     }
@@ -710,6 +698,7 @@ QVector<QPair<QString, QString> > PackageDetails::locateApplication(const QStrin
 QString PackageDetails::thumbnail(const QString &pkgName) const
 {
 #ifndef HAVE_APPSTREAM
+    Q_UNUSED(pkgName)
     return QString();
 #else
     return AppStream::instance()->thumbnail(pkgName);
@@ -719,26 +708,18 @@ QString PackageDetails::thumbnail(const QString &pkgName) const
 QString PackageDetails::screenshot(const QString &pkgName) const
 {
 #ifndef HAVE_APPSTREAM
+    Q_UNUSED(pkgName)
     return QString();
 #else
     return AppStream::instance()->screenshot(pkgName);
 #endif
 }
 
-void PackageDetails::description(const QString &packageID,
-                                 const QString &license,
-                                 PackageKit::Transaction::Group group,
-                                 const QString &detail,
-                                 const QString &url,
-                                 qulonglong size)
+void PackageDetails::description(const PackageKit::Details &details)
 {
-    kDebug() << packageID;
-    m_detailsPackageID = packageID;
-    m_detailsLicense = license;
-    m_detailsGroup = group;
-    m_detailsDetail = detail;
-    m_detailsUrl = url;
-    m_detailsSize = size;
+    kDebug() << details;
+    m_details = details;
+    m_detailsDescription = details.description();
     m_hasDetails = true;
 
 #ifdef HAVE_APPSTREAM
@@ -747,10 +728,10 @@ void PackageDetails::description(const QString &packageID,
     // and on to of the package-based model. So we can't respect the "multiple apps
     // in one package" case here.
     QList<AppStream::Application> apps;
-    apps = AppStream::instance()->applications(Transaction::packageName(packageID));
+    apps = AppStream::instance()->applications(Transaction::packageName(m_packageID));
     foreach (const AppStream::Application &app, apps) {
         if (!app.description.isEmpty()) {
-            m_detailsDetail = app.description;
+            m_detailsDescription = app.description;
             break;
         }
     }
@@ -773,9 +754,9 @@ void PackageDetails::finished()
             m_hasDetails = true;
         } else if (transaction->role() == PackageKit::Transaction::RoleGetFiles) {
             m_hasFileList = true;
-        } else if (transaction->role() == PackageKit::Transaction::RoleGetRequires) {
+        } else if (transaction->role() == PackageKit::Transaction::RoleRequiredBy) {
             m_hasRequires = true;
-        } else if (transaction->role() == PackageKit::Transaction::RoleGetDepends) {
+        } else if (transaction->role() == PackageKit::Transaction::RoleDependsOn) {
             m_hasDepends  = true;
         } else {
             return;
