@@ -72,10 +72,11 @@ public:
     PkTransactionProgressModel *progressModel;
     QWidget *parentWindow;
     QDBusObjectPath tid;
+    Transaction *transaction;
 };
 
 PkTransaction::PkTransaction(QObject *parent) :
-    Transaction(parent),
+    QObject(parent),
     d(new PkTransactionPrivate)
 {
     // for sanity we are finished till some transaction is set
@@ -95,28 +96,7 @@ PkTransaction::PkTransaction(QObject *parent) :
     d->simulateModel = 0;
     d->progressModel = new PkTransactionProgressModel(this);
     d->parentWindow = qobject_cast<QWidget*>(parent);
-
-    connect(this, SIGNAL(repoDetail(QString,QString,bool)),
-            d->progressModel, SLOT(currentRepo(QString,QString,bool)));
-    connect(this, SIGNAL(package(PackageKit::Transaction::Info,QString,QString)),
-            d->progressModel, SLOT(currentPackage(PackageKit::Transaction::Info,QString,QString)));
-    connect(this, SIGNAL(itemProgress(QString,PackageKit::Transaction::Status,uint)),
-            d->progressModel, SLOT(itemProgress(QString,PackageKit::Transaction::Status,uint)));
-
-    // Required actions
-    connect(this, SIGNAL(changed()),
-            SLOT(slotChanged()));
-    connect(this, SIGNAL(errorCode(PackageKit::Transaction::Error,QString)),
-            SLOT(slotErrorCode(PackageKit::Transaction::Error,QString)));
-    connect(this, SIGNAL(eulaRequired(QString,QString,QString,QString)),
-            SLOT(slotEulaRequired(QString,QString,QString,QString)));
-    connect(this, SIGNAL(mediaChangeRequired(PackageKit::Transaction::MediaType,QString,QString)),
-            SLOT(slotMediaChangeRequired(PackageKit::Transaction::MediaType,QString,QString)));
-    connect(this, SIGNAL(repoSignatureRequired(QString,QString,QString,QString,QString,QString,QString,PackageKit::Transaction::SigType)),
-            SLOT(slotRepoSignature(QString,QString,QString,QString,QString,QString,QString,PackageKit::Transaction::SigType)));
-
-    connect(this, SIGNAL(finished(PackageKit::Transaction::Exit,uint)),
-            SLOT(slotFinished(PackageKit::Transaction::Exit)));
+    d->transaction = 0;
 }
 
 PkTransaction::~PkTransaction()
@@ -128,17 +108,12 @@ PkTransaction::~PkTransaction()
 
 void PkTransaction::installFiles(const QStringList &files)
 {
-    if (Daemon::global()->actions() & Transaction::RoleInstallFiles) {
+    if (Daemon::global()->roles() & Transaction::RoleInstallFiles) {
         d->originalRole = Transaction::RoleInstallFiles;
         d->files = files;
         d->flags = Transaction::TransactionFlagOnlyTrusted | Transaction::TransactionFlagSimulate;
 
-        setupTransaction();
-        Transaction::installFiles(files, d->flags);
-        if (internalError()) {
-            showSorry(i18n("Failed to simulate file install"),
-                      PkStrings::daemonError(internalError()));
-        }
+        setupTransaction(Daemon::installFiles(files, d->flags));
     } else {
         showError(i18n("Current backend does not support installing files."), i18n("Error"));
     }
@@ -146,17 +121,12 @@ void PkTransaction::installFiles(const QStringList &files)
 
 void PkTransaction::installPackages(const QStringList &packages)
 {
-    if (Daemon::global()->actions() & Transaction::RoleInstallPackages) {
+    if (Daemon::global()->roles() & Transaction::RoleInstallPackages) {
         d->originalRole = Transaction::RoleInstallPackages;
         d->packages = packages;
         d->flags = Transaction::TransactionFlagOnlyTrusted | Transaction::TransactionFlagSimulate;
 
-        setupTransaction();
-        Transaction::installPackages(d->packages, d->flags);
-        if (internalError()) {
-            showSorry(i18n("Failed to simulate package install"),
-                      PkStrings::daemonError(internalError()));
-        }
+        setupTransaction(Daemon::installPackages(d->packages, d->flags));
     } else {
         showError(i18n("Current backend does not support installing packages."), i18n("Error"));
     }
@@ -164,18 +134,13 @@ void PkTransaction::installPackages(const QStringList &packages)
 
 void PkTransaction::removePackages(const QStringList &packages)
 {
-    if (Daemon::global()->actions() & Transaction::RoleRemovePackages) {
+    if (Daemon::global()->roles() & Transaction::RoleRemovePackages) {
         d->originalRole = Transaction::RoleRemovePackages;
         d->allowDeps = false; // Default to avoid dependencies removal unless simulate says so
         d->packages = packages;
         d->flags = Transaction::TransactionFlagOnlyTrusted | Transaction::TransactionFlagSimulate;
 
-        setupTransaction();
-        Transaction::removePackages(d->packages, d->allowDeps, AUTOREMOVE, d->flags);
-        if (internalError()) {
-            showSorry(i18n("Failed to simulate package removal"),
-                      PkStrings::daemonError(internalError()));
-        }
+        setupTransaction(Daemon::removePackages(d->packages, d->allowDeps, AUTOREMOVE, d->flags));
     } else {
         showError(i18n("The current backend does not support removing packages."), i18n("Error"));
     }
@@ -183,7 +148,7 @@ void PkTransaction::removePackages(const QStringList &packages)
 
 void PkTransaction::updatePackages(const QStringList &packages, bool downloadOnly)
 {
-    if (Daemon::global()->actions() & Transaction::RoleUpdatePackages) {
+    if (Daemon::global()->roles() & Transaction::RoleUpdatePackages) {
         d->originalRole = Transaction::RoleUpdatePackages;
         d->packages = packages;
         if (downloadOnly) {
@@ -193,12 +158,7 @@ void PkTransaction::updatePackages(const QStringList &packages, bool downloadOnl
             d->flags = Transaction::TransactionFlagOnlyTrusted | Transaction::TransactionFlagSimulate;
         }
 
-        setupTransaction();
-        Transaction::updatePackages(d->packages, d->flags);
-        if (internalError()) {
-            showSorry(i18n("Failed to simulate package update"),
-                      PkStrings::daemonError(internalError()));
-        }
+        setupTransaction(Daemon::updatePackages(d->packages, d->flags));
     } else {
         showError(i18n("The current backend does not support updating packages."), i18n("Error"));
     }
@@ -206,85 +166,27 @@ void PkTransaction::updatePackages(const QStringList &packages, bool downloadOnl
 
 void PkTransaction::refreshCache(bool force)
 {
-    reset();
-    Transaction::refreshCache(force);
-}
-
-void PkTransaction::setupTransaction()
-{
-    reset();
-    if (d->flags & Transaction::TransactionFlagSimulate) {
-        d->simulateModel = new PackageModel(this);
-        connect(this, SIGNAL(package(PackageKit::Transaction::Info,QString,QString)),
-                d->simulateModel, SLOT(addPackage(PackageKit::Transaction::Info,QString,QString)));
-    }
-
-#ifdef HAVE_DEBCONFKDE
-    QString _tid = tid().path();
-    QString socket;
-    // Build a socket path like /tmp/1761_edeceabd_data_debconf
-    socket = QLatin1String("/tmp") % _tid % QLatin1String("_debconf");
-    QDBusMessage message;
-    message = QDBusMessage::createMethodCall(QLatin1String("org.kde.apperd"),
-                                             QLatin1String("/"),
-                                             QLatin1String("org.kde.apperd"),
-                                             QLatin1String("SetupDebconfDialog"));
-    // Use our own cached tid to avoid crashes
-    message << qVariantFromValue(_tid);
-    message << qVariantFromValue(socket);
-    if (d->parentWindow) {
-        message << qVariantFromValue(static_cast<uint>(d->parentWindow->effectiveWinId()));
-    } else {
-        message << qVariantFromValue(0u);
-    }
-
-    if (!QDBusConnection::sessionBus().send(message)) {
-        kWarning() << "Failed to put SetupDebconfDialog message in DBus queue";
-    }
-
-    setHints(QLatin1String("frontend-socket=") % socket);
-#endif //HAVE_DEBCONFKDE
+    setupTransaction(Daemon::refreshCache(force));
 }
 
 void PkTransaction::installPackages()
 {
-    setupTransaction();
-    Transaction::installPackages(d->packages, d->flags);
-    if (internalError()) {
-        showSorry(i18n("Failed to install package"),
-                  PkStrings::daemonError(internalError()));
-    }
+    setupTransaction(Daemon::installPackages(d->packages, d->flags));
 }
 
 void PkTransaction::installFiles()
 {
-    setupTransaction();
-    Transaction::installFiles(d->files, d->flags);
-    if (internalError()) {
-        showSorry(i18np("Failed to install file",
-                        "Failed to install files", d->files.size()),
-                  PkStrings::daemonError(internalError()));
-    }
+    setupTransaction(Daemon::installFiles(d->files, d->flags));
 }
 
 void PkTransaction::removePackages()
 {
-    setupTransaction();
-    Transaction::removePackages(d->packages, d->allowDeps, AUTOREMOVE, d->flags);
-    if (internalError()) {
-        showSorry(i18n("Failed to remove package"),
-                  PkStrings::daemonError(internalError()));
-    }
+    setupTransaction(Daemon::removePackages(d->packages, d->allowDeps, AUTOREMOVE, d->flags));
 }
 
 void PkTransaction::updatePackages()
 {
-    setupTransaction();
-    Transaction::updatePackages(d->packages, d->flags);
-    if (internalError()) {
-        showSorry(i18n("Failed to update package"),
-                  PkStrings::daemonError(internalError()));
-    }
+    setupTransaction(Daemon::updatePackages(d->packages, d->flags));
 }
 
 void PkTransaction::requeueTransaction()
@@ -398,12 +300,7 @@ void PkTransaction::acceptEula()
 
     if (eula) {
         kDebug() << "Accepting EULA" << eula->id();
-        reset();
-        Transaction::acceptEula(eula->id());
-        if (internalError()) {
-            showSorry(i18n("Failed to install signature"),
-                      PkStrings::daemonError(internalError()));
-        }
+        setupTransaction(Daemon::acceptEula(eula->id()));
     } else {
         kWarning() << "something is broken, slot is bound to LicenseAgreement but signalled from elsewhere.";
     }
@@ -411,14 +308,15 @@ void PkTransaction::acceptEula()
 
 void PkTransaction::slotChanged()
 {
-    d->downloadSizeRemaining = downloadSizeRemaining();
-    d->role = role();
+    Transaction *transaction = qobject_cast<Transaction*>(sender());
+    d->downloadSizeRemaining = transaction->downloadSizeRemaining();
+    d->role = transaction->role();
 
     if (!d->jobWatcher) {
         return;
     }
 
-    QDBusObjectPath _tid = tid();
+    QDBusObjectPath _tid = transaction->tid();
     if (d->tid != _tid && !(d->flags & Transaction::TransactionFlagSimulate)) {
         d->tid = _tid;
         // if the transaction changed and
@@ -485,12 +383,7 @@ void PkTransaction::installSignature()
 
     if (repoSig)  {
         kDebug() << "Installing Signature" << repoSig->keyID();
-        reset();
-        Transaction::installSignature(repoSig->sigType(), repoSig->keyID(), repoSig->packageID());
-        if (internalError()) {
-            showSorry(i18n("Failed to install signature"),
-                      PkStrings::daemonError(internalError()));
-        }
+        setupTransaction(Daemon::installSignature(repoSig->sigType(), repoSig->keyID(), repoSig->packageID()));
     } else {
         kWarning() << "something is broken, slot is bound to RepoSig but signalled from elsewhere.";
     }
@@ -502,7 +395,8 @@ void PkTransaction::slotFinished(Transaction::Exit status)
     d->progressModel->clear();
 
     Requirements *requires = 0;
-    Transaction::Role _role = role();
+    Transaction::Role _role = qobject_cast<Transaction*>(sender())->role();
+    d->transaction = 0; // Will be deleted later
     kDebug() << status << _role;
 
     switch (_role) {
@@ -571,12 +465,9 @@ void PkTransaction::slotFinished(Transaction::Exit status)
                 connect(this, SIGNAL(files(QString,QStringList)),
                         d->launcher, SLOT(files(QString,QStringList)));
 
-                reset();
-                getFiles(d->newPackages);
+                setupTransaction(Daemon::getFiles(d->newPackages));
                 d->newPackages.clear();
-                if (!internalError()) {
-                    return; // avoid the exit code
-                }
+                return; // avoid the exit code
             } else if (_role == Transaction::RoleGetFiles &&
                        d->launcher &&
                        d->launcher->hasApplications()) {
@@ -625,8 +516,8 @@ PkTransaction::ExitStatus PkTransaction::exitStatus() const
 
 bool PkTransaction::isFinished() const
 {
-    kDebug() << status() << role();
-    return status() == Transaction::StatusFinished;
+    kDebug() << d->transaction->status() << d->transaction->role();
+    return d->transaction->status() == Transaction::StatusFinished;
 }
 
 PackageModel *PkTransaction::simulateModel() const
@@ -634,6 +525,87 @@ PackageModel *PkTransaction::simulateModel() const
     return d->simulateModel;
 }
 
+uint PkTransaction::percentage() const
+{
+    if (d->transaction) {
+        return d->transaction->percentage();
+    }
+    return 0;
+}
+
+uint PkTransaction::remainingTime() const
+{
+    if (d->transaction) {
+        return d->transaction->remainingTime();
+    }
+    return 0;
+}
+
+uint PkTransaction::speed() const
+{
+    if (d->transaction) {
+        return d->transaction->speed();
+    }
+    return 0;
+}
+
+qulonglong PkTransaction::downloadSizeRemaining() const
+{
+    if (d->transaction) {
+        return d->transaction->downloadSizeRemaining();
+    }
+    return 0;
+}
+
+Transaction::Status PkTransaction::status() const
+{
+    if (d->transaction) {
+        return d->transaction->status();
+    }
+    return Transaction::StatusUnknown;
+}
+
+Transaction::Role PkTransaction::role() const
+{
+    if (d->transaction) {
+        return d->transaction->role();
+    }
+    return Transaction::RoleUnknown;
+}
+
+bool PkTransaction::allowCancel() const
+{
+    if (d->transaction) {
+        return d->transaction->allowCancel();
+    }
+    return false;
+}
+
+Transaction::TransactionFlags PkTransaction::transactionFlags() const
+{
+    if (d->transaction) {
+        return d->transaction->transactionFlags();
+    }
+    return Transaction::TransactionFlagNone;
+}
+
+void PkTransaction::getUpdateDetail(const QString &packageID)
+{
+    setupTransaction(Daemon::getUpdateDetail(packageID));
+}
+
+void PkTransaction::getUpdates()
+{
+    setupTransaction(Daemon::getUpdates());
+}
+
+void PkTransaction::cancel()
+{
+    if (d->transaction) {
+        d->transaction->cancel();
+    }
+}
+
 void PkTransaction::setTrusted(bool trusted)
 {
     if (trusted) {
@@ -662,14 +634,101 @@ void PkTransaction::reject()
     setExitStatus(Cancelled);
 }
 
-void PkTransaction::reset()
+void PkTransaction::setupTransaction(Transaction *transaction)
 {
     // Clear the model to don't keep trash when reusing the transaction
     d->progressModel->clear();
 
-    Transaction::reset();
+    d->transaction = transaction;
+    if (!(transaction->transactionFlags() & Transaction::TransactionFlagSimulate) &&
+            transaction->role() != Transaction::RoleGetUpdates &&
+            transaction->role() != Transaction::RoleGetUpdateDetail) {
+        connect(transaction, SIGNAL(repoDetail(QString,QString,bool)),
+                d->progressModel, SLOT(currentRepo(QString,QString,bool)));
+        connect(transaction, SIGNAL(package(PackageKit::Transaction::Info,QString,QString)),
+                d->progressModel, SLOT(currentPackage(PackageKit::Transaction::Info,QString,QString)));
+        connect(transaction, SIGNAL(itemProgress(QString,PackageKit::Transaction::Status,uint)),
+                d->progressModel, SLOT(itemProgress(QString,PackageKit::Transaction::Status,uint)));
+    }
 
-    emit changed();
+    connect(transaction, SIGNAL(updateDetail(QString,QStringList,QStringList,QStringList,QStringList,QStringList,PackageKit::Transaction::Restart,QString,QString,PackageKit::Transaction::UpdateState,QDateTime,QDateTime)),
+            SIGNAL(updateDetail(QString,QStringList,QStringList,QStringList,QStringList,QStringList,PackageKit::Transaction::Restart,QString,QString,PackageKit::Transaction::UpdateState,QDateTime,QDateTime)));
+    connect(transaction, SIGNAL(package(PackageKit::Transaction::Info,QString,QString)),
+            SIGNAL(package(PackageKit::Transaction::Info,QString,QString)));
+    connect(transaction, SIGNAL(errorCode(PackageKit::Transaction::Error,QString)),
+            SIGNAL(errorCode(PackageKit::Transaction::Error,QString)));
+
+    // Required actions
+    connect(transaction, SIGNAL(allowCancelChanged()),
+            SIGNAL(allowCancelChanged()));
+    connect(transaction, SIGNAL(downloadSizeRemainingChanged()),
+            SIGNAL(downloadSizeRemainingChanged()));
+    connect(transaction, SIGNAL(elapsedTimeChanged()),
+            SIGNAL(elapsedTimeChanged()));
+    connect(transaction, SIGNAL(isCallerActiveChanged()),
+            SIGNAL(isCallerActiveChanged()));
+    connect(transaction, SIGNAL(lastPackageChanged()),
+            SIGNAL(lastPackageChanged()));
+    connect(transaction, SIGNAL(percentageChanged()),
+            SIGNAL(percentageChanged()));
+    connect(transaction, SIGNAL(remainingTimeChanged()),
+            SIGNAL(remainingTimeChanged()));
+    connect(transaction, SIGNAL(roleChanged()),
+            SIGNAL(roleChanged()));
+    connect(transaction, SIGNAL(speedChanged()),
+            SIGNAL(speedChanged()));
+    connect(transaction, SIGNAL(statusChanged()),
+            SIGNAL(statusChanged()));
+    connect(transaction, SIGNAL(transactionFlagsChanged()),
+            SIGNAL(transactionFlagsChanged()));
+    connect(transaction, SIGNAL(uidChanged()),
+            SIGNAL(uidChanged()));
+
+    connect(transaction, SIGNAL(downloadSizeRemainingChanged()),
+            SLOT(slotChanged()));
+    connect(transaction, SIGNAL(errorCode(PackageKit::Transaction::Error,QString)),
+            SLOT(slotErrorCode(PackageKit::Transaction::Error,QString)));
+    connect(transaction, SIGNAL(eulaRequired(QString,QString,QString,QString)),
+            SLOT(slotEulaRequired(QString,QString,QString,QString)));
+    connect(transaction, SIGNAL(mediaChangeRequired(PackageKit::Transaction::MediaType,QString,QString)),
+            SLOT(slotMediaChangeRequired(PackageKit::Transaction::MediaType,QString,QString)));
+    connect(transaction, SIGNAL(repoSignatureRequired(QString,QString,QString,QString,QString,QString,QString,PackageKit::Transaction::SigType)),
+            SLOT(slotRepoSignature(QString,QString,QString,QString,QString,QString,QString,PackageKit::Transaction::SigType)));
+
+    connect(transaction, SIGNAL(finished(PackageKit::Transaction::Exit,uint)),
+            SLOT(slotFinished(PackageKit::Transaction::Exit)));
+
+    if (d->flags & Transaction::TransactionFlagSimulate) {
+        d->simulateModel = new PackageModel(this);
+        connect(d->transaction, SIGNAL(package(PackageKit::Transaction::Info,QString,QString)),
+                d->simulateModel, SLOT(addPackage(PackageKit::Transaction::Info,QString,QString)));
+    }
+
+#ifdef HAVE_DEBCONFKDE
+    QString _tid = transaction->tid().path();
+    QString socket;
+    // Build a socket path like /tmp/1761_edeceabd_data_debconf
+    socket = QLatin1String("/tmp") % _tid % QLatin1String("_debconf");
+    QDBusMessage message;
+    message = QDBusMessage::createMethodCall(QLatin1String("org.kde.apperd"),
+                                             QLatin1String("/"),
+                                             QLatin1String("org.kde.apperd"),
+                                             QLatin1String("SetupDebconfDialog"));
+    // Use our own cached tid to avoid crashes
+    message << qVariantFromValue(_tid);
+    message << qVariantFromValue(socket);
+    if (d->parentWindow) {
+        message << qVariantFromValue(static_cast<uint>(d->parentWindow->effectiveWinId()));
+    } else {
+        message << qVariantFromValue(0u);
+    }
+
+    if (!QDBusConnection::sessionBus().send(message)) {
+        kWarning() << "Failed to put SetupDebconfDialog message in DBus queue";
+    }
+
+    transaction->setHints(QLatin1String("frontend-socket=") % socket);
+#endif //HAVE_DEBCONFKDE
 }
 
 void PkTransaction::showDialog(KDialog *dlg)
